#==============================================================================
# �package      DE_EPAGES::CrossSelling::UI::CrossSelling
# �state        private
#==============================================================================
package DE_EPAGES::CrossSelling::UI::CrossSelling;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::Object::API::Constants qw( POSITION_START POSITION_STEP );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias Transaction );
use DE_EPAGES::Product::API::Shop qw( GetProduct );
use DE_EPAGES::Presentation::API::PageCache qw( UpdatePageCache );
use DE_EPAGES::SEO::API::SEO qw( PerformSEOCheckAndUpdateObject );

#==============================================================================
# Event handlers
#==============================================================================
sub SaveCrossSelling {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->saveItems($Servlet, 'CrossSelling');

    return;
}

sub SaveAccessory {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->saveItems($Servlet, 'Accessory');

    return;
}

#--- do not use $self->saveItems because comparison products do not have a comment
sub SaveProductComparison {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->saveItems($Servlet, 'ProductComparison');

    return;
}

# Wizard
sub WizardSaveCrossSelling {
    my $self    = shift;
    my ($Servlet) = @_;
    $self->saveWizardItems($Servlet, 'CrossSelling');
    return;
}
sub WizardSaveAccessory {
    my $self    = shift;
    my ($Servlet) = @_;
    $self->saveWizardItems($Servlet, 'Accessory');
    return;
}
sub WizardSaveProductComparison {
    my $self    = shift;
    my ($Servlet) = @_;
    $self->saveWizardItems($Servlet, 'ProductComparison');
    return;
}


sub AddCrossSellingProductsFromTray {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->addProductsFromTray($Servlet, 'CrossSelling');
}

sub AddAccessoryProductsFromTray {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->addProductsFromTray($Servlet, 'Accessory');
}

sub AddComparisonProductsFromTray {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->addProductsFromTray($Servlet, 'ProductComparison');
}

sub RemoveCrossSellingProducts {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->removeProducts($Servlet, 'CrossSelling');
}

sub RemoveAllCrossSellingProducts {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;
    my $aCrossSelling = $Product->get('CrossSelling');
    return unless scalar @$aCrossSelling;

    #delete objects by using wrapper
    $self->SUPER::deleteObjectListWrapper($aCrossSelling);

    #trigger refresh of SEO check and update cache
    PerformSEOCheckAndUpdateObject($Product);
    UpdatePageCache($Product);
}

sub RemoveAccessoryProducts {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->removeProducts($Servlet, 'Accessory');
}

sub RemoveComparisonProducts {
    my $self    = shift;
    my ($Servlet) = @_;

    $self->removeProducts($Servlet, 'ProductComparison');
}

sub saveItems {
    my $self    = shift;
    my ($Servlet, $Name) = @_;

    my $Product = $Servlet->object;
    my $Form = $Servlet->form;

    my $Shop = $Product->getSite();
    my $Alias  = $Form->value('NewProduct');
    my $TargetProduct;
    $TargetProduct = GetProduct($Product->getSite(), $Alias, 1) if defined $Alias and $Alias =~ /\S/;

    if (not defined $TargetProduct and defined $Alias and $Alias =~ /\S/) {
        $Servlet->form->addFormError({
            'Reason'=> 'ERROR_INVALIDPRODUCT',
            'Form'  => $Name,
            'Name'  => 'NewProduct',
        });
    }

    $self->addProduct($Product, $TargetProduct, $Name) if defined $TargetProduct and $TargetProduct->id != $Product->id;
    $self->addProduct($TargetProduct, $Product, $Name) if $Form->value('BothWayReference')
                                                          and defined $TargetProduct
                                                          and $TargetProduct->id != $Product->id;
    my $Folder  = $Product->folder($Name);

    # for unity we use an extra form because of language switch and not existing comments
    # first save the CrossSellingBoxHeadline
    if ($Shop->get('IsUnityRegistered')) {
        $Shop->set({"UnityCrossSellingBoxHeadline" => $Form->value('CrossSellingBoxHeadline') },$Servlet->form->value('LanguageID'));
        $Name = 'UnityCrossSelling'; # for fetching the correct form
    }

    my $hValues = $Servlet->form->form($Product, $Name);

    my @List = map {
                       {
                          'Object' => $Shop->loadObject($_->{'PositionObjectID'}, 'Object'),
                          'Position' => $_->{'Position'}
                       }
                   } @{$hValues->{'Items'}};

    # sort list by position
    @List = sort { $a->{'Position'} <=> $b->{'Position'} } @List;

    Transaction( sub {
        my $Position = POSITION_START;
        foreach my $hItem ( @List ) {
            my $Object = $hItem->{Object};

            # security check: <$Name> item products belong to the
            # <$Name> folder of this product
            die 'Invalid parent object' unless $Object->get('Parent')->id == $Folder->id;

            if ($Name eq 'Accessory' or 'CrossSelling') {
                # update comment in all languages
                my @Comments = grep { $_->{'CommentObjectID'} == $Object->id } @{$hValues->{'Comments'}};
                foreach my $hComment ( @Comments ) {
                    $Object->set({'Comment' => $hComment->{'Comment'} }, $hComment->{'LanguageID'} );
                }
            }

            # update Position in steps of 10
            $Object->set({'Position' => $Position });
            $Position += POSITION_STEP;
        }
    });
    $Form->executeFormError;
    UpdatePageCache($Product);

    return;
}

sub saveWizardItems {
    my $self    = shift;
    my ($Servlet, $Name) = @_;

    my $Product = $Servlet->object;
    my $Form = $Servlet->form;

    my $hValues = $Servlet->form->form($Product, $Name.'Wizard');

    if (not defined $hValues->{$Name.'Items'}) {
        $Servlet->form->addFormError({
            'Reason'=> 'ERROR_NOITEMS',
            'Form'  => $Name,
            'Name'  => 'NewProduct',
        });
        return $Form->executeFormError;
    }
    my @Items = @{$hValues->{$Name.'Items'}};

    my @AllComments;
    if($hValues->{$Name.'Comments'}) {
       @AllComments  = @{$hValues->{$Name.'Comments'}};
    }

    foreach my $Item (@Items) {
        my $Alias  = $Item->{$Name.'NewProduct'};
        my $TargetProduct;
        $TargetProduct = GetProduct($Product->getSite(), $Alias, 1) if defined $Alias and $Alias =~ /\S/;

        if (not defined $TargetProduct and defined $Alias and $Alias =~ /\S/) {
            $Servlet->form->addFormError({
                'Reason'=> 'ERROR_INVALIDPRODUCT',
                'Form'  => $Name,
                'Name'  => 'NewProduct',
            });
        }
        my $ResultObject;
        $ResultObject = $self->addProduct($Product, $TargetProduct, $Name) if defined $TargetProduct and $TargetProduct->id != $Product->id;
        next if not defined $ResultObject;

        my $Folder  = $Product->folder($Name);
        die 'Invalid parent object' unless $ResultObject->get('Parent')->id == $Folder->id;

        if ($Name eq 'Accessory' or 'CrossSelling') {
            # update comment in all languages
            my @Comments = grep { $_->{$Name.'CommentTargetAlias'} eq $Alias} @AllComments;
            foreach my $hComment ( @Comments ) {
                $ResultObject->set( {'Comment' => $hComment->{$Name.'Comment'} },  $hComment->{$Name.'LanguageID'} );
            }
        }
        $ResultObject->set( {'Position' => $Item->{$Name.'Position'} });
    }

    $Form->executeFormError;
    UpdatePageCache($Product);

    return;
}

sub addProductsFromTray {
    my $self = shift;
    my ($Servlet, $Name) = @_;

    my $Product = $Servlet->object;
    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    my $Objects = $Tray->objectsByClass(LoadClassByAlias('Product'));
    my $Folder = $Product->folder($Name);
    my $aChildren = $Folder->children;

    # provide 'exists' check by object id
    my %TargetProductIDs = map { $_->get('TargetProduct')->id => undef } @$aChildren;

    my $NextPosition = scalar @$aChildren
        ? POSITION_STEP + $aChildren->[-1]->get('Position')
        : POSITION_START;
    Transaction( sub {
        foreach my $TrayObject (@$Objects) {
            # don't add an object twice
            next if exists $TargetProductIDs{ $TrayObject->id };

            # don't add a self reference
            next if $TrayObject->id == $Product->id;

            # add a new product to the folder
            $Folder->insertChild($Name, {
                Alias => $Folder->nextAlias,
                Product => $Product,
                TargetProduct => $TrayObject,
                Position => $NextPosition,
            });
            $NextPosition += POSITION_STEP;
        }
    });

    # create reference in both ways if selected
    if($Servlet->form->value('BothWayReference')) {
        Transaction( sub {
            foreach my $TrayObject (@$Objects) {

                $Folder = $TrayObject->folder($Name);
                my $aTrayObjectChildren = $Folder->children;

                # provide 'exists' check by object id
                my %ReverseTargetProductIDs = map { $_->get('TargetProduct')->id => undef } @$aTrayObjectChildren;

                # don't add an object twice
                next if exists $ReverseTargetProductIDs{ $Product->id };

                # don't add a self reference
                next if $TrayObject->id == $Product->id;

                # add a new product to the folder
                $Folder->insertChild($Name, {
                    Alias => $Folder->nextAlias,
                    Product => $TrayObject,
                    TargetProduct => $Product,
                    Position => scalar @$aTrayObjectChildren ? POSITION_START + $aTrayObjectChildren->[-1]->get('Position') : POSITION_STEP,
                });
            }
        });
    }

    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    UpdatePageCache($Product);
    return;
}

sub removeProducts {
    my $self = shift;
    my ($Servlet, $Name) = @_;

    my $Product = $Servlet->object;
    my $Class = LoadClassByAlias($Name);
    $self->SUPER::DeleteListedObjects( $Servlet, $Class );
    $Product->folder($Name)->renumberChildren;

    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    UpdatePageCache($Product);
    return;
}

sub addProduct {
    my $self = shift;
    my ($Product, $TargetProduct, $Name) = @_;

    my $Folder = $Product->folder($Name);
    my $aChildren = $Folder->children;

    # provide 'exists' check by object id
    my %TargetProductIDs = map { $_->get('TargetProduct')->id => undef } @$aChildren;
    my $ResultObject;

    Transaction( sub {
        # don't add an object twice
        if(exists $TargetProductIDs{ $TargetProduct->id }) {
            foreach my $Child (@{$aChildren}) {
                my $ReferencedProduct = $Child->get('TargetProduct');
                if($ReferencedProduct->id == $TargetProduct->id) {
                    $ResultObject = $Child;
                    last;
                }
            }
            return;
        }

        # add a new product to the folder
        $ResultObject = $Folder->insertChild($Name, {
            Alias => $Folder->nextAlias,
            Product => $Product,
            TargetProduct => $TargetProduct,
            Position => 9999,
        });
        $Product->folder($Name)->renumberChildren;
    });

    # trigger refresh of SEO check
    PerformSEOCheckAndUpdateObject($Product);

    UpdatePageCache($Product);

    return $ResultObject;
}

sub _DeleteMappingFromProduct {
    # EPG-29824: can't delete cross-selling assignment on the "References" tab
    # Product.getAttributes returns Product objects instead of Accessory/CrossSelling/ProductComparison objects
    # use getObjects() directly to fetch proper objects

    my ($Product, $RefClassName, $aRefIDs) = @_;
    return unless scalar @$aRefIDs;

    my $TargetObjects = LoadClassByAlias($RefClassName)->getObjects('ByTargetProductID', [$Product->id]);
    foreach my $Reference ( @$TargetObjects ) {
        foreach my $ProductID (@$aRefIDs) {
            my $RefProduct = $Reference->get('Product');
            if ($RefProduct->id == $ProductID) {
                $Reference->delete;
                $RefProduct->folder($RefClassName)->renumberChildren;
            }
        }
    }

    return;
}

sub DeleteMapping {
    my $self = shift;
    my ($Servlet) = @_;

    my $Product = $Servlet->object;

    # do nothing if no objects are supplied
    return if ! $Servlet->params('ListObjectID');

    my @ProductIDs = $Servlet->form->value('ListObjectID');

    # delete accessory mapping
    my @AccessoryProductIDs = map { /(\d+)$/ } grep { $_ =~ /accessory/ } @ProductIDs;
    _DeleteMappingFromProduct($Product, 'Accessory', \@AccessoryProductIDs);

    # delete manual cross-selling/complementory mapping
    my @ComplementoryProductIDs = map { /(\d+)$/ } grep { $_ =~ /complementory/ } @ProductIDs;
    _DeleteMappingFromProduct($Product, 'CrossSelling', \@ComplementoryProductIDs);

    # delete product comparison mapping
    my @ComparisonProductIDs = map { /(\d+)$/ } grep { $_ =~ /comparison/ } @ProductIDs;
    _DeleteMappingFromProduct($Product, 'ProductComparison', \@ComparisonProductIDs);

    # delete BundleProductMap for selected BundleProductID
    my @BundleProductIDs = map { /(\d+)$/ } grep { $_ =~ /bundle/ } @ProductIDs;
    if (scalar @BundleProductIDs) {
        my $Maps = LoadClassByAlias('BundleProductMap')->getObjects('ByBundledProductID', [$Product->id]);
        $_->delete foreach grep { ElementOf($_->get('BundleProduct')->id, @BundleProductIDs) } @$Maps;
    }

    PerformSEOCheckAndUpdateObject($Product);

    return;
}

sub UnityView{
    my $self = shift;
    my ($Servlet) = @_;

    # enable language switch
    $Servlet->vars('ShowLanguageSwitch', 1);

    $self->View($Servlet);
}

1;
