#==============================================================================
# package      DE_EPAGES::Customer::API::Constants
# state        private
#------------------------------------------------------------------------------
# description  Some constants used with CSV Export Import
#==============================================================================
package DE_EPAGES::Customer::API::Constants;
use base Exporter;
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use strict;
our @EXPORT = qw(
    $CSV_SPECIAL_CUSTOMERITEMS
    $CSV_SPECIAL_CUSTOMERITEMS_UNITY
    @CSV_WARNING_CODES_CUSTOMER
    @UNITY_REGISTRATION_SET
    $UNITY_REGISTRATION_SET_ADDITIONAL
    REGISTRATION_FIELDS_MAX_LENGTH
    SPAM_FIELD_VALUES
);

use vars qw ( $CSV_SPECIAL_CUSTOMERITEMS
              $CSV_SPECIAL_CUSTOMERITEMS_UNITY
              $UNITY_REGISTRATION_SET_ADDITIONAL
              @CSV_WARNING_CODES_CUSTOMER
            );
use Readonly;



#========================================================================================
# global       $CSV_SPECIAL_CUSTOMERITEMS_UNITY | csv export import constants <ul>
#                   <li>0 - do not export this attributes</li>
#                   <li>1 - export this attributes, also if not exportable</li>
#               </ul> | ref.array.string
#========================================================================================
$CSV_SPECIAL_CUSTOMERITEMS_UNITY = {
    0 => [ qw( IsMandateIDValid
               CustomerGroup
               TaxArea
               TaxModel
               Comment
               IsDoOrderAllowed
               IsRatingInvitationAllowed
               IsHtmlEMailAllowed
               User.ERPSystem
               User.CurrencyID
               User.IsActivated
               User.IsHtmlEMailAllowed
               ) ],
    1 => [ qw( BillingAddress.Country
               ShippingAddress.Country ) ],
};

#========================================================================================
# global       $CSV_SPECIAL_CUSTOMERITEMS | csv export import constants <ul>
#                   <li>0 - do not export this attributes</li>
#                   <li>1 - export this attributes, also if not exportable</li>
#               </ul> | ref.array.string
#========================================================================================
$CSV_SPECIAL_CUSTOMERITEMS = {
    0 => [ qw( GUID
               Inherit
               CreationDate
               Class
               Parent
               Site
               Position
               EbayShopEnvironment
               EbayUserName
               BillingAddress
               ShippingAddress
               Shop
               FilePathPart
               BillingAddress.Alias
               BillingAddress.Class
               BillingAddress.Inherit
               BillingAddress.CountryID
               BillingAddress.CreationDate
               BillingAddress.GUID
               BillingAddress.FilePathPart
               BillingAddress.Position
               User.BillingAddress
               User.CountLoginFailure
               User.CreationDate
               User.DeleteConfirmation
               User.GUID
               User.Inherit
               User.LanguageID
               User.Password
               User.Position
               User.LastLoginFailure
               User.ShippingAddress
               User.ShowFunctionHelp
               User.ShowSidebar
               User.SidebarMenuName
               User.FilePathPart
               User.IsLoginCookieAllowed
               User.GotEkomiEmail
               User.PagerEntries
               User.ReminderQuestion
               User.ChallengePhrase
               BillingAddress.IsColissimoOutOfHomeDelivery
               ShippingAddress.IsColissimoOutOfHomeDelivery
               BillingAddress.AvaTax_Hash
               ShippingAddress.AvaTax_Hash
               User.LastPositiveCreditCheck
               User.IsEmailVerified
               BillingAddress.IsPickupAddress
               ShippingAddress.IsPickupAddress
               BillingAddress.SelectedShippingPickupStation
               ShippingAddress.SelectedShippingPickupStation
               MandateIDLastOrderDate
               IsMandateIDValid
               BillingAddress.DHLPackstationId
               ShippingAddress.DHLPackstationId
               ) ],
    1 => [ qw( BillingAddress.Country ) ],
};

#========================================================================================
# global   REGISTRATION_FIELDS_MAX_LENGTH
#           | Max length of fields in the customer registration form
#           | Hash of field name to respective max character length.
#           | ref.hash.array.int
# example  my $MaxLength = REGISTRATION_FIELDS_MAX_LENGTH->{'FirstName'}
#========================================================================================
sub REGISTRATION_FIELDS_MAX_LENGTH {
    my $Address = LoadClassByAlias('Address');
    my @aFields = @{$Address->directAttributes()};
    my %hNameToLengthHash;

    foreach my $Field (@aFields) {
        next unless defined $Field->get('Length');
        $hNameToLengthHash{$Field->get('Alias')} = $Field->get('Length');
    }
    return \%hNameToLengthHash;
}

@CSV_WARNING_CODES_CUSTOMER = qw(
    NO_CUSTOMERITEMS
    NO_RECIPIENTS
    FORMAT_NOT_EMAILADDRESS
    SHOW_COMPARE_ADDRESSES
    FORMAT_NOT_DATETIME
    FORMAT_NOT_TIME
    CUSTOMERGOUP_DOESNT_EXIST
    CUSTOMER_EMPTY_ALIAS
    TAXAREA_DOESNT_EXIST
    NO_PREDEF_ATTRIBUTE
    WRONG_PREDEF
    INVALID_USER_LOCALE
    LOGIN_EXISTS
    NO_SHIPPINGADDRESS_FOR_CUSTOMERS
    UNKNOWN_COUNTRYCODE
    VATIDIsNotFromSameCountry
    VATIDIsNotValid
    ZIPCODE_NOT_VALID
    GENDER_NOT_VALID
);

#========================================================================================
# global       @UNITY_REGISTRATION_SET | Available fields for the registration set of
#                 Unity shops (UNITY-4187) | ref.array.string
#========================================================================================
Readonly our @UNITY_REGISTRATION_SET => qw(
    Company
    VATID
    Salutation
    FirstName
    LastName
    Street
    Street2
    Zipcode
    City
    CountryID
    State
    Birthday
    FiscalCode
    Phone
    PhoneCell
    EMail
);

#========================================================================================
# global       $UNITY_REGISTRATION_SET_ADDITIONAL | Country dependend additional fields for the 
#               registration set of Unity shops (AD-9183) | ref.hash.array.string
#========================================================================================
$UNITY_REGISTRATION_SET_ADDITIONAL = {
    IT => [ qw( EMailBusiness ) ],
};

#========================================================================================
# global       SPAM_FIELD_VALUES | returns spamFieldValues
#========================================================================================
sub SPAM_FIELD_VALUES() { ## no critic qw(RequireCommentForExportedFunctions)
    return EpagesConfig()->get('SpamRegistrationFields', 'SpamFieldValues');
}


1;
